# -*- coding: utf-8 -*-
"""
This module contains shared code generation tools for the ufc::*_integral classes.
"""

# Copyright (C) 2008-2012 Martin Sandve Alnes and Simula Resarch Laboratory
#
# This file is part of SyFi.
#
# SyFi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# SyFi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with SyFi. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2008-08-13
# Last changed: 2012-05-25

from sfc.common.output import sfc_debug, sfc_error, sfc_assert, sfc_warning

from sfc.codegeneration.codeformatting import indent

class IntegralCGBase(object):
    def __init__(self, itgrep):
        self.itgrep = itgrep
        self.options = self.itgrep.formrep.options.code.integral

    def hincludes(self):
        l = []
        return l

    def cincludes(self):
        l = []
        return l

    def local_includes(self):
        l = []
        return l

    def generate_code_dict(self):
        # Calling these first so we can use the analysis results in other functions
        tabulate_tensor = indent(self.gen_tabulate_tensor())
        tabulate_tensor_quadrature = indent(self.gen_tabulate_tensor_quadrature())
        vars = {
            "classname"                  : self.itgrep.classname,
            "constructor"                : indent(self.gen_constructor()),
            "constructor_arguments"      : indent(self.gen_constructor_arguments()),
            "initializer_list"           : indent(self.gen_initializer_list()),
            "destructor"                 : indent(self.gen_destructor()),
            "members"                    : indent(self.gen_members()),
            "tabulate_tensor"            : tabulate_tensor,
            "tabulate_tensor_quadrature" : tabulate_tensor_quadrature,
            }
        return vars

    def generate_support_code(self):
        return ""

    def gen_constructor(self):
        return ""

    def gen_constructor_arguments(self):
        return ""

    def gen_initializer_list(self):
        return ""

    def gen_destructor(self):
        return ""

    def gen_members(self):
        return ""

    def gen_tabulate_tensor(self):
        return 'throw std::runtime_error("Not implemented.");'

    def gen_tabulate_tensor_quadrature(self):
        return 'throw std::runtime_error("Not implemented.");'
