# -*- coding: utf-8 -*-
"""
This module contains representation classes for integrals.
"""

# Copyright (C) 2012-2012 Martin Sandve Alnes and Simula Resarch Laboratory
#
# This file is part of SyFi.
#
# SyFi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# SyFi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with SyFi. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2012-05-29
# Last changed: 2012-05-29

#import SyFi
import swiginac
#import uflacs

from sfc.common import sfc_assert, sfc_warning, sfc_debug, sfc_error
from ufl.permutation import compute_indices
#from sfc.common.utilities import indices_subset

from sfc.representation.integralrepresentationbase import IntegralRepresentationBase

class UflacsIntegralRepresentation(IntegralRepresentationBase):
    def __init__(self, integrals, formrep, on_facet):
        IntegralRepresentationBase.__init__(self, integrals, formrep, on_facet)

        # Shape of element tensor
        A_shape = []
        for i in range(self.formrep.rank):
            element = self.formrep.formdata.elements[i]
            rep = self.formrep.element_reps[element]
            A_shape.append(rep.local_dimension)
        self.A_shape = tuple(A_shape)

        # All permutations of element tensor indices
        # TODO: Not same for interior_facet_integrals, move to subclasses
        self.indices = compute_indices(self.A_shape)

        self.num_quadrature_points = (formrep.facet_quad_rules[0].num_points
                                      if on_facet
                                      else formrep.quad_rule.num_points)

class UflacsCellIntegralRepresentation(UflacsIntegralRepresentation):
    def __init__(self, integrals, formrep):
        UflacsIntegralRepresentation.__init__(self, integrals, formrep, False)

class UflacsExteriorFacetIntegralRepresentation(UflacsIntegralRepresentation):
    def __init__(self, integrals, formrep):
        UflacsIntegralRepresentation.__init__(self, integrals, formrep, True)

class UflacsInteriorFacetIntegralRepresentation(UflacsIntegralRepresentation):
    def __init__(self, integrals, formrep):
        UflacsIntegralRepresentation.__init__(self, integrals, formrep, True)


CellIntegralRepresentation = UflacsCellIntegralRepresentation
ExteriorFacetIntegralRepresentation = UflacsExteriorFacetIntegralRepresentation
InteriorFacetIntegralRepresentation = UflacsInteriorFacetIntegralRepresentation
